@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 0.3
+;; Version: 0.5
 ;; Package-Requires: ((emacs "27.1"))
 ;; Homepage: https://github.com/minad/tempel
 
@@ -22,13 +22,13 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
 ;; Tempel implements a simple template/snippet system. The template
 ;; format is compatible with the template format of the Emacs Tempo
-;; library. Your templates are stored in the `tempel-file' (by default
+;; library. Your templates are stored in the `tempel-path' (by default
 ;; the file "templates" in the `user-emacs-directory'). Bind the
 ;; commands `tempel-complete', `tempel-expand' or `tempel-insert' to
 ;; some keys in your user configuration. You can jump with the keys M-{
@@ -48,9 +48,19 @@
   :group 'editing
   :prefix "tempel-")
 
-(defcustom tempel-file (expand-file-name "templates" user-emacs-directory)
-  "Path to the template file."
-  :type 'string)
+(defcustom tempel-path (expand-file-name "templates" user-emacs-directory)
+  "A file or a list of template files.
+The file paths can contain wildcards, e.g.,
+\"~/.config/emacs/templates/*/*.eld\", which matches all
+`lisp-data-mode' files in the subdirectories of the templates
+directory."
+  :type '(choice string (repeat string)))
+
+(defcustom tempel-trigger-prefix nil
+  "Trigger string prefixes the template names.
+The trigger prefix must be entered first before the template name to
+trigger completion."
+  :type '(choice (const nil) string))
 
 (defcustom tempel-mark
   #(" " 0 1 (display (space :width (1)) face cursor))
@@ -72,12 +82,17 @@ nil or a new template element, which is subsequently evaluated."
   :type 'hook)
 
 (defcustom tempel-template-sources
-  (list #'tempel-file-templates)
+  (list #'tempel-path-templates)
   "List of template sources.
 A source can either be a function or a variable symbol. The functions
 must return a list of templates which apply to the buffer or context."
   :type 'hook)
 
+(defcustom tempel-auto-reload t
+  "Reload templates when files specified by `tempel-path' change.
+If a file is modified, added or removed, reload the templates."
+  :type 'boolean)
+
 (defface tempel-field
   '((((class color) (min-colors 88) (background light))
      :background "#fdf0ff" :foreground "#541f4f")
@@ -102,11 +117,8 @@ must return a list of templates which apply to the buffer or context."
     (t :inherit highlight :slant italic))
   "Face used for default values.")
 
-(defvar tempel--file-templates nil
-  "Templates loaded from the `tempel-file'.")
-
-(defvar tempel--file-modified nil
-  "Modification time of `tempel-file' at the last load.")
+(defvar tempel--path-templates nil
+  "Templates loaded from the `tempel-path'.")
 
 (defvar tempel--history nil
   "Completion history used by `tempel-insert'.")
@@ -131,6 +143,7 @@ may be named with `tempel--name' or carry an evaluatable Lisp expression
     (define-key map [remap backward-paragraph] #'tempel-previous)
     (define-key map [remap forward-paragraph] #'tempel-next)
     ;; Use concrete keys because of org mode
+    (define-key map "\M-\r" #'tempel-done)
     (define-key map "\M-{" #'tempel-previous)
     (define-key map "\M-}" #'tempel-next)
     (define-key map [M-up] #'tempel-previous)
@@ -178,14 +191,14 @@ REGION are the current region bouns"
     (when-let* ((sym (intern-soft name))
                 (template (alist-get sym templates)))
       (tempel--delete-word name)
+      (when tempel-trigger-prefix
+        (tempel--delete-word tempel-trigger-prefix))
       (tempel--insert template region))))
 
 (defun tempel--range-modified (ov &rest _)
   "Range overlay OV modified."
   (when (and (not tempel--inhibit-hooks) (= (overlay-start ov) (overlay-end ov)))
-    (let ((st (overlay-get ov 'tempel--range)))
-      (setq tempel--active (cons st (delq st tempel--active)))
-      (tempel--disable))))
+    (tempel--disable (overlay-get ov 'tempel--range))))
 
 (defun tempel--field-modified (ov after beg end &optional _len)
   "Update field overlay OV.
@@ -257,7 +270,8 @@ If OV is alive, move it."
 (defun tempel--field (st &optional name init)
   "Add template field to ST.
 NAME is the optional field name.
-INIT is the optional initial input."
+INIT is the optional initial input.
+Return the added field."
   (let ((ov (make-overlay (point) (point))))
     (push ov (car st))
     (when name
@@ -277,10 +291,12 @@ INIT is the optional initial input."
       (overlay-put ov 'face 'tempel-default)
       (overlay-put ov 'tempel--default
                    (if (string-match-p ": \\'" init) 'end 'start)))
-    (tempel--synchronize-fields st ov)))
+    (tempel--synchronize-fields st ov)
+    ov))
 
 (defun tempel--form (st form)
-  "Add new template field evaluating FORM to ST."
+  "Add new template field evaluating FORM to ST.
+Return the added field."
   (let ((beg (point)))
     (condition-case nil
         (insert (eval form (cdr st)))
@@ -289,15 +305,22 @@ INIT is the optional initial input."
     (let ((ov (make-overlay beg (point) nil t)))
       (overlay-put ov 'face 'tempel-form)
       (overlay-put ov 'tempel--form form)
-      (push ov (car st)))))
+      (push ov (car st))
+      ov)))
+
+(defmacro tempel--protect (&rest body)
+  "Protect BODY, catch errors."
+  `(with-demoted-errors "Tempel Error: %S"
+     ,@body))
 
 (defun tempel--element (st region elt)
   "Add template ELT to ST given the REGION."
   (pcase elt
     ('nil)
     ('n (insert "\n"))
-    ('n> (insert "\n") (indent-according-to-mode))
-    ('> (indent-according-to-mode))
+    ;; `indent-according-to-mode' fails sometimes in Org. Ignore errors.
+    ('n> (insert "\n") (tempel--protect (indent-according-to-mode)))
+    ('> (tempel--protect (indent-according-to-mode)))
     ((pred stringp) (insert elt))
     ('& (unless (or (bolp) (save-excursion (re-search-backward "^\\s-*\\=" nil t)))
           (insert "\n")))
@@ -309,10 +332,15 @@ INIT is the optional initial input."
     (`(l . ,lst) (dolist (e lst) (tempel--element st region e)))
     ((or 'p `(,(or 'p 'P) . ,rest)) (apply #'tempel--placeholder st rest))
     ((or 'r 'r> `(,(or 'r 'r>) . ,rest))
-     (if (not region) (apply #'tempel--placeholder st rest)
+     (if (not region)
+         (when-let (ov (apply #'tempel--placeholder st rest))
+           (unless rest
+             (overlay-put ov 'tempel--quit t)))
        (goto-char (cdr region))
        (when (eq (or (car-safe elt) elt) 'r>)
          (indent-region (car region) (cdr region) nil))))
+    ;; TEMPEL EXTENSION: Quit template immediately
+    ('q (overlay-put (tempel--field st) 'tempel--quit t))
     (_ (if-let (ret (run-hook-with-args-until-success 'tempel-user-elements elt))
            (tempel--element st region ret)
          ;; TEMPEL EXTENSION: Evaluate forms
@@ -321,7 +349,8 @@ INIT is the optional initial input."
 (defun tempel--placeholder (st &optional prompt name noinsert)
   "Handle placeholder element and add field with NAME to ST.
 If NOINSERT is non-nil do not insert a field, only bind the value to NAME.
-PROMPT is the optional prompt/default value."
+PROMPT is the optional prompt/default value.
+If a field was added, return it."
   (setq prompt
         (cond
          ((and (stringp prompt) noinsert) (read-string prompt))
@@ -329,7 +358,7 @@ PROMPT is the optional prompt/default value."
          ;; TEMPEL EXTENSION: Evaluate prompt
          (t (eval prompt (cdr st)))))
   (if noinsert
-      (setf (alist-get name (cdr st)) prompt)
+      (progn (setf (alist-get name (cdr st)) prompt) nil)
     (tempel--field st name prompt)))
 
 (defun tempel--insert (template region)
@@ -340,7 +369,7 @@ PROMPT is the optional prompt/default value."
     (eval (plist-get plist :pre) 'lexical)
     ;; TODO do we want to have the ability to reactivate snippets?
     (unless (eq buffer-undo-list t)
-      (push (list 'apply #'tempel--disable) buffer-undo-list))
+      (push '(apply tempel--disable) buffer-undo-list))
     (setf (alist-get 'tempel--active minor-mode-overriding-map-alist) tempel-map)
     (save-excursion
       ;; Split existing overlays, do not expand within existing field.
@@ -359,25 +388,36 @@ PROMPT is the optional prompt/default value."
         (push range (car st))
         (overlay-put range 'modification-hooks (list #'tempel--range-modified))
         (overlay-put range 'tempel--range st)
+        (overlay-put range 'tempel--post (plist-get plist :post))
         ;;(overlay-put range 'face 'region) ;; TODO debug
         (push st tempel--active)))
     (cond
      ((cl-loop for ov in (caar tempel--active)
                never (overlay-get ov 'tempel--field))
       (goto-char (overlay-end (caaar tempel--active)))
-      (tempel--disable)) ;; Disable right away
+      (tempel--done)) ;; Finalize right away
      ((cl-loop for ov in (caar tempel--active)
                never (and (overlay-get ov 'tempel--field)
                           (eq (point) (overlay-start ov))))
-      (tempel-next 1))) ;; Jump to first field
-    (eval (plist-get plist :post) 'lexical)))
+      (tempel-next 1))))) ;; Jump to first field
 
 (defun tempel--save ()
-  "Save template file buffer."
-  (when-let (buf (get-file-buffer tempel-file))
-    (with-current-buffer buf
-      (when (and (buffer-modified-p) (y-or-n-p (format "Save file %s? " tempel-file)))
-        (save-buffer buf)))))
+  "Prompt to save modified files in `tempel-path'."
+  (cl-loop
+   with all = nil
+   for (file . _ts) in (car tempel--path-templates) do
+   (when-let (buf (get-file-buffer file))
+     (with-current-buffer buf
+       (when (and (buffer-modified-p)
+                  (pcase (or all (read-answer
+                                  (format "Save file %s? " file)
+                                  '(("yes" ?y "save the file")
+                                    ("no"  ?n "skip the file")
+                                    ("all" ?! "save all modified files"))))
+                    ("yes" t)
+                    ("no" nil)
+                    ("all" (setq all "all"))))
+         (save-buffer buf))))))
 
 (defun tempel--file-read (file)
   "Load templates from FILE."
@@ -400,26 +440,43 @@ PROMPT is the optional prompt/default value."
           (push `(,(nreverse modes) ,(nreverse plist) . ,(nreverse templates)) result)))
       result)))
 
-(defun tempel-file-templates ()
-  "Load the templates defined in `tempel-file'."
-  (let ((mod (time-convert (file-attribute-modification-time
-                            (file-attributes tempel-file))
-                           'integer)))
-    (unless (equal tempel--file-modified mod)
-      (setq tempel--file-templates (tempel--file-read tempel-file)
-            tempel--file-modified mod)))
+(defun tempel-path-templates ()
+  "Return templates defined in `tempel-path'.
+Additionally, save any files in `tempel-template-sources' that have been
+modified since the last time this function was called.
+This is meant to be a source in `tempel-template-sources'."
+  (when (or (not tempel--path-templates) tempel-auto-reload)
+    (let* ((files
+            (cl-loop for path in (if (listp tempel-path) tempel-path (list tempel-path))
+                     nconc (file-expand-wildcards path t)))
+           (timestamps
+            (cl-loop
+             for f in files collect
+             (cons f (time-convert
+                      (file-attribute-modification-time
+                       (file-attributes f))
+                      'integer)))))
+      (unless (equal (car tempel--path-templates) timestamps)
+        (setq tempel--path-templates (cons timestamps
+                                           (mapcan #'tempel--file-read files))))))
   (cl-loop
-   for (modes plist . templates) in tempel--file-templates
-   if (and
-       (cl-loop for m in modes
-                thereis (or (derived-mode-p m) (eq m #'fundamental-mode)))
-       (or (not (plist-member plist :condition))
-           (save-excursion
-             (save-restriction
-               (save-match-data
-                 (eval (plist-get plist :condition) 'lexical))))))
+   for (modes plist . templates) in (cdr tempel--path-templates)
+   if (tempel--condition-p modes plist)
    append templates))
 
+(defun tempel--condition-p (modes plist)
+  "Return non-nil if one of MODES matches and the PLIST condition is satisfied."
+  (and
+   (cl-loop
+    for m in modes thereis
+    (or (eq m #'fundamental-mode)
+        (derived-mode-p m)))
+   (or (not (plist-member plist :when))
+       (save-excursion
+         (save-restriction
+           (save-match-data
+             (eval (plist-get plist :when) 'lexical)))))))
+
 (defun tempel--templates ()
   "Return templates for current mode."
   (let (result)
@@ -485,7 +542,12 @@ PROMPT is the optional prompt/default value."
   (cl-loop for i below (abs arg) do
            (if-let (next (tempel--find arg)) (goto-char next)
              (tempel-done)
-             (cl-return))))
+             (cl-return)))
+  ;; If the current field is marked as "quitting", disable its
+  ;; containing template right away.
+  (when-let* ((ov (tempel--field-at-point))
+              ((overlay-get ov 'tempel--quit)))
+    (tempel--done (overlay-get ov 'tempel--field))))
 
 (defun tempel-previous (arg)
   "Move ARG fields backward and quit at the beginning."
@@ -505,15 +567,18 @@ PROMPT is the optional prompt/default value."
 (defun tempel-abort ()
   "Abort template insertion."
   (interactive)
-  ;; TODO abort only the topmost template?
   (when-let ((beg (tempel--beginning))
              (end (tempel--end)))
-    (tempel-done)
+    ;; TODO abort only the topmost template?
+    (while tempel--active (tempel--disable))
     (delete-region beg end)))
 
-(defun tempel--disable ()
-  "Disable last template."
-  (when-let (st (pop tempel--active))
+(defun tempel--disable (&optional st)
+  "Disable template ST, or last template."
+  (if st
+      (setq tempel--active (delq st tempel--active))
+    (setq st (pop tempel--active)))
+  (when st
     (mapc #'delete-overlay (car st))
     (unless tempel--active
       (setq minor-mode-overriding-map-alist
@@ -524,7 +589,17 @@ PROMPT is the optional prompt/default value."
   "Template completion is done."
   (interactive)
   ;; TODO disable only the topmost template?
-  (while tempel--active (tempel--disable)))
+  (while tempel--active (tempel--done)))
+
+(defun tempel--done (&optional st)
+  "Finalize template ST, or last template."
+  (let ((st (or st (car tempel--active)))
+        (buf (current-buffer)))
+    ;; Ignore errors in post expansion to ensure that templates can be
+    ;; terminated gracefully.
+    (tempel--protect (eval (overlay-get (caar st) 'tempel--post) (cdr st)))
+    (with-current-buffer buf
+      (tempel--disable st))))
 
 (defun tempel--interactive (capf)
   "Complete with CAPF."
@@ -541,6 +616,18 @@ The completion table specifies the category `tempel'."
         '(metadata (category . tempel))
       (complete-with-action action templates str pred))))
 
+(defun tempel--prefix-bounds ()
+  "Return prefix bounds."
+  (if tempel-trigger-prefix
+      (let ((end (point))
+            (beg (save-excursion
+                   (search-backward tempel-trigger-prefix
+                                    (line-beginning-position) 'noerror))))
+        (when (and beg (save-excursion
+                         (not (re-search-backward "\\s-" beg 'noerror))))
+          (cons (+ beg (length tempel-trigger-prefix)) end)))
+    (bounds-of-thing-at-point 'symbol)))
+
 ;;;###autoload
 (defun tempel-expand (&optional interactive)
   "Expand exactly matching template name at point.
@@ -549,7 +636,7 @@ If INTERACTIVE is nil the function acts like a capf."
   (if interactive
       (tempel--interactive #'tempel-expand)
     (when-let* ((templates (tempel--templates))
-                (bounds (bounds-of-thing-at-point 'symbol))
+                (bounds (tempel--prefix-bounds))
                 (name (buffer-substring-no-properties
                        (car bounds) (cdr bounds)))
                 (sym (intern-soft name))
@@ -566,16 +653,20 @@ If INTERACTIVE is nil the function acts like a capf."
 If INTERACTIVE is nil the function acts like a capf."
   (interactive (list t))
   (if interactive
-      (tempel--interactive #'tempel-complete)
-    (when-let (templates (tempel--templates))
-      (let* ((region (tempel--region))
-             (bounds (or (and (not region) (bounds-of-thing-at-point 'symbol))
-                         (cons (point) (point)))))
+      (progn
+        (when (and tempel-trigger-prefix (not (tempel--prefix-bounds)))
+          (insert tempel-trigger-prefix))
+        (tempel--interactive #'tempel-complete))
+    (let ((region (tempel--region)))
+      (when-let ((templates (tempel--templates))
+                 (bounds (or (and (not region) (tempel--prefix-bounds))
+                             (and (not tempel-trigger-prefix) (cons (point) (point))))))
         (list (car bounds) (cdr bounds)
               (tempel--completion-table templates)
               :exclusive 'no
               :company-kind (lambda (_) 'snippet)
               :exit-function (apply-partially #'tempel--exit templates region)
+              :company-prefix-length (and tempel-trigger-prefix t)
               :annotation-function
               (and tempel-complete-annotation
                    (apply-partially #'tempel--annotate
@@ -638,15 +729,17 @@ If called interactively, select a template with `completing-read'."
   (when tempel-abbrev-mode
     (let ((table (make-abbrev-table)))
       (dolist (template (tempel--templates))
-        (define-abbrev table (symbol-name (car template)) 'Template
-          (apply-partially #'tempel--abbrev-hook (symbol-name (car template)) (cdr template))
-          :system t))
+        (let* ((name (symbol-name (car template)))
+               (hook (make-symbol name)))
+          (fset hook (apply-partially #'tempel--abbrev-hook name (cdr template)))
+          (put hook 'no-self-insert t)
+          (define-abbrev table name 'Template hook :system t)))
       (setq-local abbrev-minor-mode-table-alist
                   (cons `(tempel-abbrev-mode . ,table)
                         abbrev-minor-mode-table-alist)))))
 
 ;;;###autoload
-(define-globalized-minor-mode tempel-global-abbrev-mode tempel-abbrev-mode
+(define-globalized-minor-mode global-tempel-abbrev-mode tempel-abbrev-mode
   tempel--abbrev-on :group 'tempel)
 
 (defun tempel--abbrev-on ()
